#!/bin/bash
#
# Zabbix Slack AlertScript
#
# Encoding: UTF-8N / LF
#
# Requirements:
#   Zabbix 2.0 or later:
#   curl
#
# Installation:
#   sudo cp slack-notify.sh /etc/zabbix/alertscripts/
#   sudo chmod 755 /etc/zabbix/alertscripts/*.sh
#
# See also:
#   Custom alertscripts: https://www.zabbix.com/documentation/2.0/manual/config/notifications/media/script

export PATH=/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin
export LANG=en_US.UTF-8
readonly CMDNAME=$(basename $0)
readonly CUR_DIR=$(cd $(dirname $0);pwd)
OPT_CONF_FILE=$CUR_DIR/$(basename $CMDNAME .sh).conf
OPT_SLACK_WEBHOOK_URL='' # example: https://hooks.slack.com/services/T00000000/B00000000/XXXXXXXXXXXXXXXXXXXXXXXX
OPT_SLACK_CHANNEL=''  # example: '#channel'
OPT_SLACK_USERNAME='zabbix-webhook'
OPT_DEBUG=0

if [ -f "$OPT_CONF_FILE" ]; then
	source $OPT_CONF_FILE
fi

echo_debug() {
	if [ "$OPT_DEBUG" == "1"  ]; then
		echo -e "[DEBUG] $1" | logger -i -t $CMDNAME
	fi
}

# Slack color code: https://api.slack.com/docs/message-attachments
#   - color: good, warning, danger, #439FE0
# Zabbix Trigger severity: https://www.zabbix.com/documentation/2.0/manual/config/triggers/severity
choose_color() {
	local to="$1"
	local subject="$2"
	local body="$3"
	local color="#DBDBDB"

	if [[ "$body" =~ 'status: OK' ]]; then
		color="good"
	elif [[ "$body" =~ 'severity: Information' ]]; then
		color="#D6F6FF"
	elif [[ "$body" =~ 'severity: Warning' ]]; then
		color="#FFF6A5"
	elif [[ "$body" =~ 'severity: Average' ]]; then
		color="#FFB689"
	elif [[ "$body" =~ 'severity: High' ]]; then
		color="#FF9999"
	elif [[ "$body" =~ 'severity: Disaster' ]]; then
		color="#FF3838"
	else
		color="#DBDBDB" # Not classified
	fi

	echo "$color"
}

post_message() {
	local to="$1"
	local subject="$2"
	local body="$3"
	
	local slack_text="$subject"
	local attachments_text="$body"
	local attachments_color=$(choose_color "$@")
	local payload=$(slack_payload "$to" "$OPT_SLACK_USERNAME" "$slack_text" "$attachments_text" "$attachments_color")
	slack_post "$OPT_SLACK_WEBHOOK_URL" "$payload"
}

slack_payload(){
	local channel=${1//\"/\\\"}
	local username=${2//\"/\\\"}
	local text=${3//\"/\\\"}
	local attachments_text=${4//\"/\\\"}
	local attachments_color=${5//\"/\\\"}

	cat << EOS
{
  "channel": "${channel}",
  "username": "${username}",
  "text": "${text}",
  "attachments": [
    {
      "color": "${attachments_color}",
      "text": "${attachments_text}"
    }
  ]
}
EOS
}

slack_post(){
	local url="$1"
	local payload="$2"
	
    # --data-urlencode is not supported on CentOS 5.x with curl 7.15.5.
	local result=$( curl -s -S \
	     --retry 1 \
	     --retry-max-time 3 \
	     --max-time 3 \
             -w ":%{http_code}" \
	     -X POST "$url" \
	     -d "$payload" 2>&1 )

	if [ "$?" != "0" -o "$result" != "ok:200" ]; then
		echo "Failed to slack_post(). result:$result url:$url payload:$payload" | logger -i -t $CMDNAME
	fi
}

main() {
	OPT_SLACK_CHANNEL="$1"
	local subject="$2"
	local body="$3"

	echo_debug "OPT_SLACK_CHANNEL: $OPT_SLACK_CHANNEL\nsubject: $subject\nbody: $body\n"

	if [ $# -le 0 ]; then
		usage_exit
	fi

	if [ "$OPT_SLACK_WEBHOOK_URL" == "" ]; then
		echo "OPT_SLACK_WEBHOOK_URL is empty" 1>&2
		exit 1
	fi

	if [ "$OPT_SLACK_CHANNEL" == "" ]; then
		echo "OPT_SLACK_CHANNEL is empty" 1>&2
		exit 1
	fi
	
	post_message "$OPT_SLACK_CHANNEL" "$subject" "$body"
}

usage_exit() {
    echo -e "Usage: $CMDNAME [OPTIONS] <to> <subject> <body>

OPTIONS
  <to> : zabbix to (require)
    example: #channel or @username

  <subject> : zabbix subject (require)

  <body> : zabbix body (require)

  -w : Slack Incoming Webhooks URL (require)
    example: https://hooks.slack.com/services/T00000000/B00000000/XXXXXXXXXXXXXXXXXXXXXXXX

  -c : Slack channel (optional)
    example: #channel

  -u : Slack username (optional)
    example: zabbix-webhook

  -h : help
" 1>&2
    exit 1
}

while getopts "w:,c:,u:,h" OPT
do
  case $OPT in
    "w" )
        OPT_SLACK_WEBHOOK_URL="$OPTARG"
        ;;
    "c" )
        OPT_SLACK_CHANNEL="$OPTARG"
        ;;
    "u" )
        OPT_SLACK_USERNAME="$OPTARG"
        ;;
    "h" )
        usage_exit
        ;;
    * )
        echo "Unknown option(${OPT}) used." 1>&2
        exit 1
        ;;
  esac
done
 
shift $(expr $OPTIND - 1)
 
main "$@"
